module Control.Addressbook.Streaming where

import qualified Data.Text as T

import Conduit
import qualified Data.Conduit.Binary as CB
import qualified Data.Conduit.Combinators as C
import qualified Data.Conduit.Text as CT

import Data.Email
import Data.Email.Header
       (Header(..))

import System.IO
       (stdin)

import qualified Data.Foldable as F

import qualified Data.Set as S

combine :: (MonadUnliftIO m, MonadResource m, MonadThrow m, MonadIO m) => ConduitM FilePath Header m ()
combine = await >>= \case
  Nothing -> pure ()
  Just path -> (CB.sourceFile path .| parseEmail) >> combine

run :: IO ()
run = do
  x <- runResourceT $ runConduit stream
  F.for_ x print
  where
    separate = \case
      From x -> [x]
      To xs -> F.toList xs
    stream =
      CB.sourceHandle stdin
        .| CT.decode CT.utf8
        .| CT.lines
        .| C.map T.unpack
        .| combine
        .| C.concatMap separate
        .| C.foldMap (S.singleton)
